#
# SPDX-FileCopyrightText: Copyright (c) provide.io llc. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#

"""Data models for tfbrowser entities."""

from .module import Module, ModuleVersion  # Example, adjust to actual class names
from .provider import Provider, ProviderVersion  # Example, adjust to actual class names
from .version import VersionInfo  # Example, adjust to actual class names

__all__ = [
    "Module",
    "ModuleVersion",
    "Provider",
    "ProviderVersion",
    "VersionInfo",
]

# 🥣🔬🔚
